from pyrogram import filters
from pyrogram.errors import MessageNotModified
from pyrogram.types import CallbackQuery, Message

from . import app
from askpyro.helpers import ikb, ikb2 


async def gen_button_kb(q: Message or CallbackQuery):
    return ikb(
        [
            [
                ("ɪɴʟɪɴᴇ 1", "buttons.back"),
            ],
            [("ɪɴʟɪɴᴇ 2", "Callback.2")],
        ],
    )


async def gen_button_kb2(m):
    return ikb2(
        [
            [
                ("ɪɴʟɪɴᴇ 1", "Callback1"),
                ("ɪɴʟɪɴᴇ 2", "Callback2"),
            ],
            [("ɪɴʟɪɴᴇ 3", "Callback3")],
        ],
        True,
        "back_callback",
    )


@app.on_message(filters.command("buttons"))
async def buttonkb_(c: app, m: Message):
    await m.reply_text(
        f"ᴇᴀsʏ ɪɴʟɪɴᴇ ʙᴜᴛᴛᴏɴ ᴜsɪɴɢ ᴀsᴋ-ᴘʏʀᴏ",
        reply_markup=(await gen_button_kb(m)),
    )
    return
  
  
@app.on_callback_query(filters.regex("^buttons."))
async def buttonkb_CB(c: app, q: CallbackQuery):
    data = q.data.split(".")[1]
    if data == "back":
        try:
            await q.message.edit_text(
                text=f"ᴇᴀsʏ ɪɴʟɪɴᴇ ʙᴜᴛᴛᴏɴ ᴜsɪɴɢ ᴀsᴋ-ᴘʏʀᴏ ",
                reply_markup=(await gen_button_kb(q.message)),
            )
        except MessageNotModified:
            pass
        await q.answer()
        return



from pyrogram import filters, Client
from pyrogram.types import Message
from askpyro import Askclient

app = Client("AskPyro", api_id=API_ID, api_hash=API_HASH, bot_token=BOT_TOKEN)

read = Askclient(app)

@app.on_message(filters.command("start"))  
async def start_(c:app, m: Message):
    answer = await read.ask(m,"ʜᴏᴡ ᴀʀᴇ ʏᴏᴜ ʙʀᴏ ?")
    await answer.reply(f"ʏᴏᴜʀ ᴀɴsᴡᴇʀ ɪs {ans.text}")

    
asu.start()



from pyrogram.types import ForceReply,Message,CallbackQuery
import asyncio
import logging
from .errors import errors
from pyrogram import Client,filters
from pyrogram.filters import Filter

class Askclient():
  def __init__(self,
               client : Client,
               clear : bool = True,
               timeout : int = 40,
               stop_cmd : str = "/cancel",
               placeholder : str = "Send required information...",
               msg_limit : int = 100
               ):
      if not isinstance(client, Client):
          print("Parameter client must be pyrogram Client object")
          return errors.basic
      if not isinstance(timeout, int):
          print("Parameter timeout must be Integer")
          return errors.basic
      if not isinstance(stop_cmd, str):
          print("Parameter stop_cmd must be str")
          return errors.basic
      if not isinstance(placeholder, str):
          print("Parameter placeholder must be str")
          return errors.basic
      if not isinstance(clear, bool):
          print("Parameter clear must be Boolean (True or False)")
          return errors.basic
      self.client : Client = client
      self.timeout : int = timeout
      self.stop_cmd : str = stop_cmd
      self.placeholder : str = placeholder
      self.msg_limit : int = msg_limit
      self.delete_msg : bool = clear

  async def ask(self,
                msg : Message or CallbackQuery,
                text : str,
                alert : bool = False,
                filter : Filter = False
                ):
     if not isinstance(alert,bool):
        print("Parameter alert must be Boolean (True or False)")
        return errors.basic
     if (alert and (not isinstance(msg,CallbackQuery))) or ((not alert) and (not isinstance(msg,Message))):
        if alert:
         print("Parameter msg must be CallbackQuery object")
         return errors.basic
        else:
         print("Parameter msg must be Message object")
         return errors.basic      
     if not isinstance(text,str):
        print("Parameter text must be str")
        return errors.basic
     if not (isinstance(filter,bool) or isinstance(filter,Filter)):
        print("Parameter filter must be pyrogram filters object")
        return errors.basic
     try:
         try:
             if msg.from_user:
                 anonymous = False
                 user_id = msg.from_user.id
             else:
                 anonymous = True
                 user_id = msg.sender_chat.id
         except Exception as e:
             logging.info(e)
             user_id = None
         if user_id:
             if alert:
                 uv = await self.client.send_message(chat_id=msg.message.chat.id,
                                                     text=text,
                                                     reply_markup=ForceReply(
                                                                             selective = True,
                                                                             placeholder = self.placeholder
                                                                            )
                                                     )
             else:
                 uv = await msg.reply(text = text,
                                         reply_to_message_id = msg.id,
                                         reply_markup=ForceReply(selective = True,
                                                                 placeholder = self.placeholder
                                                                 )
                                         )
             ans = await asyncio.wait_for(Askclient.wait(self, uv, anonymous, user_id, filter),
                                          timeout=self.timeout
                                          )
         else:
             return errors.unknown
     except Exception as e:
      logging.info("Time Out!")
      ans = errors.timeout
     try:
      if self.delete_msg:
        await uv.delete()
     except:
      pass
     return ans


  async def wait(self,uv,anonymous,user_id,filter):
      while True:
          try:
              ids = [uv.id + x for x in range(self.msg_limit)]
              ids.pop(0)
              for ans in (await self.client.get_messages(uv.chat.id, ids)):
                  ans.error = errors.unknown
                  try:
                      if anonymous:
                          user_id2 = ans.sender_chat.id
                      else:
                          user_id2 = ans.from_user.id
                  except:
                      user_id2 = "Nonne"
                  if filter:
                      is_user = True
                  else:
                      is_user = True if (user_id2 == user_id) else False
                  if ans and (not ans.empty) and (ans.text == self.stop_cmd) and is_user:
                      ans = errors.cancel
                      break
                  if ans and (not ans.empty) and is_user:
                      if filter:
                          fil = await filter(self.client, ans)
                      else:
                          fil = await filters.text(self.client, ans)
                      if fil:
                          ans.error = ans.cancel = ans.timeout = ans.anon = ans.unknown = False
                          break
              if (not ans.error) or (ans == errors.cancel):
                  break
          except Exception as e:
              logging.info(e)
              break
      return ans



"""
Askpyro - A add-on for Pyrogram
Copyright (C) 2023  Abishnoi. <https://github.com/Abishnoi69>
This file is part of pyromod.
Askpyro is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Askpyro is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License.

along with Askpyro.  If not, see <https://www.gnu.org/licenses/>.
"""

from .helpers import ikb, bki, ntb, btn, kb, kbtn, array_chunk, force_reply, ikb2
from .parser import cleanhtml, escape_markdown, mention_html, mention_markdown



from pyrogram import filters, Client
from pyrogram.types import Message
from askpyro import Askclient

app = Client("AskPyro", api_id=API_ID, api_hash=API_HASH, bot_token=BOT_TOKEN)

read = Askclient(app)

@app.on_message(filters.command("start"))  
"""https://github.com/Abishnoi69/askpyro/wiki/pyrogram-filters"""
async def filters_(c:app, m:Message):
    answer = await read.ask(m, "ʜᴏᴡ ᴀʀᴇ ʏᴏᴜ ʙʀᴏ ?", filter=(filters.text & filters.group))
    print(answer.text)

app.run()





from pyrogram import filters, Client
from pyrogram.types import CallbackQuery
from pyrocon import Askclient

app = Client("AskPyro", api_id=API_ID, api_hash=API_HASH, bot_token=BOT_TOKEN)

read = Askclient(app)

@app.on_callback_query()  
async def callback(_, msg: CallbackQuery):
    answer = await read.ask(msg, "ʜᴏᴡ ᴀʀᴇ ʏᴏᴜ ʙʀᴏ ?", alert=True)
    await answer.reply(f"ʏᴏᴜʀ ᴀɴsᴡᴇʀ ɪs {ans.text}")
    
app.run()




__license__ = "GNU Lesser General Public License v3.0 (LGPL-3.0)"
__copyright__ = "Copyright (C) 2012-present Abishnoi <https://github.com/Abishnoi69>"

from .helpers import *
from .conversation import Askclient
from .errors import errors





from pyrogram.types import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    KeyboardButton,
    ReplyKeyboardMarkup,
    ForceReply,
)


def ikb(rows=[]):
    lines = []
    for row in rows:
        line = []
        for button in row:
            button = (
                btn(button, button) if type(button) == str else btn(*button)
            )  # InlineKeyboardButton
            line.append(button)
        lines.append(line)
    return InlineKeyboardMarkup(inline_keyboard=lines)
    # return {'inline_keyboard': lines}


def btn(text, value, type="callback_data"):
    return InlineKeyboardButton(text, **{type: value})
    # return {'text': text, type: value}


# The inverse of above
def bki(keyboard):
    lines = []
    for row in keyboard.inline_keyboard:
        line = []
        for button in row:
            button = ntb(button)  # btn() format
            line.append(button)
        lines.append(line)
    return lines
    # return ikb() format


def ntb(button):
    for btn_type in [
        "callback_data",
        "url",
        "switch_inline_query",
        "switch_inline_query_current_chat",
        "callback_game",
    ]:
        value = getattr(button, btn_type)
        if value:
            break
    button = [button.text, value]
    if btn_type != "callback_data":
        button.append(btn_type)
    return button
    # return {'text': text, type: value}


def kb(rows=[], **kwargs):
    lines = []
    for row in rows:
        line = []
        for button in row:
            button_type = type(button)
            if button_type == str:
                button = KeyboardButton(button)
            elif button_type == dict:
                button = KeyboardButton(**button)

            line.append(button)
        lines.append(line)
    return ReplyKeyboardMarkup(keyboard=lines, **kwargs)


kbtn = KeyboardButton


def force_reply(selective=True):
    return ForceReply(selective=selective)


def array_chunk(input, size):
    return [input[i : i + size] for i in range(0, len(input), size)]
  
  
  
  
  
  




def ikb2(rows=None, back=False, todo="start_back"):
    
    #rows = pass the rows
    #back - if want to make back button
    #todo - callback data of back button
    
    if rows is None:
        rows = []
    lines = []
    try:
        for row in rows:
            line = []
            for button in row:
                btn_text = button.split(".")[1].upper()
                button = btn(btn_text, button)  # InlineKeyboardButton
                line.append(button)
            lines.append(line)
    except AttributeError:
        for row in rows:
            line = []
            for button in row:
                button = btn(*button)  # Will make the kb which don't have "." in them
                line.append(button)
            lines.append(line)
    except TypeError:
        # make a code to handel that error
        line = []
        for button in rows:
            button = btn(*button)  # InlineKeyboardButton
            line.append(button)
        lines.append(line)
    if back:
        back_btn = [(btn("« ʙᴀᴄᴋ", todo))]
        lines.append(back_btn)
    return InlineKeyboardMarkup(inline_keyboard=lines)
  
